# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: api/stats/team_stats.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from ...api.stats import stats_pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='api/stats/team_stats.proto',
  package='api.stats',
  serialized_pb=_b('\n\x1a\x61pi/stats/team_stats.proto\x12\tapi.stats\x1a\x15\x61pi/stats/stats.proto\"`\n\tTeamStats\x12)\n\npossession\x18\x01 \x01(\x0b\x32\x15.api.stats.Possession\x12(\n\nhit_counts\x18\x02 \x01(\x0b\x32\x14.api.stats.HitCounts')
  ,
  dependencies=[stats_pb2.DESCRIPTOR,])
_sym_db.RegisterFileDescriptor(DESCRIPTOR)




_TEAMSTATS = _descriptor.Descriptor(
  name='TeamStats',
  full_name='api.stats.TeamStats',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='possession', full_name='api.stats.TeamStats.possession', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='hit_counts', full_name='api.stats.TeamStats.hit_counts', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=64,
  serialized_end=160,
)

_TEAMSTATS.fields_by_name['possession'].message_type = stats_pb2._POSSESSION
_TEAMSTATS.fields_by_name['hit_counts'].message_type = stats_pb2._HITCOUNTS
DESCRIPTOR.message_types_by_name['TeamStats'] = _TEAMSTATS

TeamStats = _reflection.GeneratedProtocolMessageType('TeamStats', (_message.Message,), dict(
  DESCRIPTOR = _TEAMSTATS,
  __module__ = 'api.stats.team_stats_pb2'
  # @@protoc_insertion_point(class_scope:api.stats.TeamStats)
  ))
_sym_db.RegisterMessage(TeamStats)


# @@protoc_insertion_point(module_scope)
